#ifndef TCPSERVERTHREAD_CC
#define TCPSERVERTHREAD_CC

//
// simple tcp server thread class
// 
// Markus Geisler <geisler@primusnetz.de>
//

#include "TCPserverThread.h"

// constructor
TCPserverThread::TCPserverThread()
{
	syslog(7,"[child] starting new server thread\n");
	status = RUNNING;
	inputBuffer = (char *) malloc(MAXBUFFERSIZE);
	bufferpos = 0;
	myId = 0;
};

// destructor
TCPserverThread::~TCPserverThread()
{
	syslog(7,"[child %i] destroying server thread\n",myId);
};

// get data from supplied socket
void TCPserverThread::run()
{
	int n;
	int i;
	char *tmp_char;
	char rcv_msg[MAXSIZE];

	while(1)
	{
		// once connected, wait for data
		n = recv(mysocket, rcv_msg, MAXSIZE, 0);
		if(n)
		{
			rcv_msg[n]='\0';
			for(i=bufferpos;i<bufferpos+(int)strlen(rcv_msg);i++)
			{
				assert(i<MAXBUFFERSIZE);
				if(i<MAXBUFFERSIZE)
				{
					inputBuffer[i] = rcv_msg[i-bufferpos];
				}
				else
				{
					perror("Inputbuffer overflow!");
				}
			}
			bufferpos = i;
			inputBuffer[bufferpos] = '\0';

			syslog(7,"[child %i] received message from peer: (size: %i),%s\n",myId,n,rcv_msg);

			n = 0;

			if(strstr(inputBuffer,"quit"))
			{
				syslog(7,"[child %i] client disconnected with quit\n",myId);
				close(mysocket);
				break;
			}
			else
			if(strstr(inputBuffer,"\n\0"))
			{
				// End-Tag
				syslog(7,"[child %i] client send END-Tag!\n",myId);

				// inputBuffer verarbeiten!
				// in - durch semaphoren geschtzten - speicherbereich schreiben
				pthread_mutex_lock(mymutex);
				myFeedbackBuffer->insertData(inputBuffer,myId);
				tmp_char = myFeedbackBuffer->getData();
				syslog(7,"[child %i] writing input buffer to feedback buffer: %s\n",myId,inputBuffer);
				send(mysocket,tmp_char,strlen(tmp_char),0);
				free(tmp_char);
				pthread_mutex_unlock(mymutex);

				// flush buffer and reset index
				inputBuffer[0]='\0';
				bufferpos = 0;
				i = 0;
			}
		}
		else
		{
			syslog(7,"[child %i] client disconnected\n",myId);
			break;
		}
	}

	status = NOTRUNNING;
	pthread_exit(NULL);
};

// set new socket
void TCPserverThread::setSocket(int newsocket)
{
	mysocket = newsocket;
}
// set new mutex
void TCPserverThread::setMutex(pthread_mutex_t *mutex)
{
	mymutex = mutex;
}

// set new id
void TCPserverThread::setId(int id)
{
	myId = id;
}

// set feedbackbuffer
void TCPserverThread::setFeedbackBuffer(FeedbackBuffer *tmpBuffer)
{
	myFeedbackBuffer = tmpBuffer;
}

// is running?
int TCPserverThread::isRunning()
{
	return(status);
}

// wrapper member, wegen pthread
void *TCPserverThread::wrapper(void *object)
{
	TCPserverThread *myThread = (TCPserverThread *)object;
	myThread->run();
	return(myThread);
}

// get Buffer Data
char *TCPserverThread::getData()
{
	return(strdup(inputBuffer));
}

#endif
